#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Graphical Extension for Users who prefer a GUI over CLI."""

import sys

from PyQt6.QtGui import QIcon
from PyQt6.QtWidgets import (QApplication, QMainWindow, QStatusBar, QTabWidget,
                             QVBoxLayout)

from torrentfileQt.checkTab import CheckWidget
from torrentfileQt.createTab import CreateWidget
from torrentfileQt.editorTab import EditorWidget
from torrentfileQt.infoTab import InfoWidget
from torrentfileQt.menu import MenuBar
# from torrentfileQt.qss import (mainWindowSheet, statusBarSheet, tabBarSheet,
#                                tabSheet)
from torrentfileQt.qss import stylesheet


class Window(QMainWindow):
    """Window MainWindow of GUI extension interface.

    Subclass:
        QMainWindow (QWidget): PyQt6 QMainWindow
    """

    def __init__(self, parent=None, app=None):
        """Constructor for Window class.

        Args:
            parent (QWidget, optional): The current Widget's parent.
                Defaults to None.
            app (QApplication, optional): Controls the GUI application.
                Defaults to None.
        """
        super().__init__(parent=parent)
        self.app = app
        self.menubar = MenuBar(parent=self)
        self.statusbar = QStatusBar(parent=self)
        self.icon = QIcon("./assets/favicon.png")
        self.setObjectName("Mainwindow")
        self.setWindowTitle("Torrentfile Tools")
        self.setWindowIcon(self.icon)
        self.setMenuBar(self.menubar)
        self.setStatusBar(self.statusbar)
        # self.setStyleSheet(mainWindowSheet + statusBarSheet)
        self.setStyleSheet(stylesheet)
        self.resize(750, 650)
        self._setupUI()

    def _setupUI(self):
        """Internal function for setting up UI elements."""
        self.central = TabWidget(parent=self)
        self.centralLayout = QVBoxLayout()
        self.central.setLayout(self.centralLayout)
        self.setCentralWidget(self.central)
        self.statusbar.setObjectName("statusbar")
        self.menubar.setObjectName("menubar")
        self.central.setObjectName("centralTabWidget")
        self.centralLayout.setObjectName("centralLayout")


class TabWidget(QTabWidget):
    """Qt Widget subclass for the tab widget."""

    def __init__(self, parent=None):
        """Construct Tab Widget for MainWindow.

        Args:
            parent (`QWidget`, deault=None): QMainWindow
        """
        super().__init__(parent=parent)
        self.window = parent
        self.createWidget = CreateWidget(parent=self)
        self.checkWidget = CheckWidget(parent=self)
        self.infoWidget = InfoWidget(parent=self)
        self.editorWidget = EditorWidget(parent=self)
        # self.setStyleSheet(tabBarSheet + tabSheet)
        self.addTab(self.createWidget, "Create Torrent")
        self.addTab(self.checkWidget, "Re-Check Torrent")
        self.addTab(self.infoWidget, "Torrent Info")
        self.addTab(self.editorWidget, "Torrent Editor")
        self.createWidget.setObjectName("createTab")
        self.checkWidget.setObjectName("checkTab")
        self.infoWidget.setObjectName("infoTab")
        self.editorWidget.setObjectName("editorTab")


class Application(QApplication):
    """QApplication Widget."""

    def __init__(self, args=None):
        """Constructor for main application backend.

        Args:
            args (`list`, optional): argument list passed to window.
                Defaults to None.
        """
        self.args = args if args else sys.argv
        super().__init__(self.args)


def start():  # pragma: no cover
    """Entrypoint for program."""
    app = Application()
    window = Window(parent=None, app=app)
    window.show()
    sys.exit(app.exec())


def alt_start():
    """Entrypoint for testing scripts."""
    app = Application()
    window = Window(parent=None, app=app)
    window.show()
    return window, app


if __name__ == "__main__":
    start()  # pragma: no cover



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Module for stylesheets."""

stylesheet = (
    """
    * {
        background-color: #19232D;
        color: #FFFFFF;
        padding: 0px;
        margin: 0px;
        border-width: 2px;
        border-style: solid;
    }
    QMenu,
    QTreeWidget,
    QTreeWidget::item,
    QTableWidget,
    QTableWidget::item,
    QHeaderView::section,
    QTableView QTableCornerButton::section,
    QLineEdit,
    QTextEdit,
    QComboBox,
    QComboBox:editable,
    QComboBox QAbstractItemView,
    QProgressBar,
    QScrollBar {
        border-width: 2px;
        border-style: solid;
        border-radius: 2px;
        padding: 0px;
        margin: 0px;
    }
    QHeaderView::section,
    QTableView QTableCornerButton::section,
    QLineEdit,
    QTextEdit,
    QComboBox:editable,
    QComboBox,
    QComboBox QAbstractItemView,
    QProgressBar,
    QScrollBar {
        border-color: #f61;
        selection-background-color: #2f4f4f;
    }
    QLabel,
    QLabel:disabled {
        border-width: 1px;
        background-color: transparent;
        padding: 0px;
        border-color: transparent;
        selection-background-color: #2f4f4f;
    }
    QWidget {
        background-color: #39434d
    }
    QLineEdit {
        background: qradialgradient(cx: 0.3, cy: -0.4,
        fx: 0.3, fy: -0.4,
        radius: 1.35, stop: 0 #444, stop: 1 #111);
        padding: 4px;
        border-radius: 8px;
    }
    QAbstractScrollArea {
        background-color: #19232D;
        border: 1px solid #455364;
        border-radius: 4px;
        padding: 2px;
        color: #E0E1E3;
    }
    QAbstractScrollArea:disabled {
        color: #9DA9B5;
    }
    QScrollArea QWidget QWidget:disabled {
        background-color: #19232D;
    }
    QScrollBar:horizontal {
        height: 16px;
        margin: 2px 16px 2px 16px;
        border: 1px solid #455364;
        border-radius: 4px;
        background-color: #19232D;
    }
    QScrollBar:vertical {
        background-color: #19232D;
        width: 16px;
        margin: 16px 2px 16px 2px;
        border: 1px solid #455364;
        border-radius: 4px;
    }
    QScrollBar::handle:horizontal {
        background-color: #60798B;
        border: 1px solid #455364;
        border-radius: 4px;
        min-width: 8px;
    }
    QScrollBar::handle:horizontal:hover {
        background-color: #346792;
        border: #346792;
        border-radius: 4px;
        min-width: 8px;
    }
    QScrollBar::handle:horizontal:focus {
        border: 1px solid #1A72BB;
    }
    QScrollBar::handle:vertical {
        background-color: #60798B;
        border: 1px solid #455364;
        min-height: 8px;
        border-radius: 4px;
    }
    QScrollBar::handle:vertical:hover {
        background-color: #346792;
        border: #346792;
        border-radius: 4px;
        min-height: 8px;
    }
    QScrollBar::handle:vertical:focus {
        border: 1px solid #1A72BB;
    }
    QScrollBar::add-line:horizontal {
        margin: 0px 0px 0px 0px;
        height: 12px;
        width: 12px;
        subcontrol-position: right;
        subcontrol-origin: margin;
    }
    QScrollBar::add-line:horizontal:hover, QScrollBar::add-line:horizontal:on {
        height: 12px;
        width: 12px;
        subcontrol-position: right;
        subcontrol-origin: margin;
    }
    QScrollBar::add-line:vertical {
        margin: 3px 0px 3px 0px;
        height: 12px;
        width: 12px;
        subcontrol-position: bottom;
        subcontrol-origin: margin;
    }
    QScrollBar::add-line:vertical:hover, QScrollBar::add-line:vertical:on {
        height: 12px;
        width: 12px;
        subcontrol-position: bottom;
        subcontrol-origin: margin;
    }
    QScrollBar::sub-line:horizontal {
        margin: 0px 3px 0px 3px;
        height: 12px;
        width: 12px;
        subcontrol-position: left;
        subcontrol-origin: margin;
    }
    QScrollBar::sub-line:horizontal:hover, QScrollBar::sub-line:horizontal:on {
        height: 12px;
        width: 12px;
        subcontrol-position: left;
        subcontrol-origin: margin;
    }
    QScrollBar::sub-line:vertical {
        margin: 3px 0px 3px 0px;
        height: 12px;
        width: 12px;
        subcontrol-position: top;
        subcontrol-origin: margin;
    }
    QScrollBar::sub-line:vertical:hover, QScrollBar::sub-line:vertical:on {
        height: 12px;
        width: 12px;
        subcontrol-position: top;
        subcontrol-origin: margin;
    }
    QTabBar::tab {
        border-style: outset;
        border-color: #f51;
        border-width: 3px;
        font-size: 11pt;
        color: #e9e7e6;
        padding-left: 8px;
        padding-right: 8px;
        margin-right: 2px;
        padding-bottom: 2px;
        background-color: #444;
    }
    QTabBar::tab:hover {
        color: #ffffff;
    }
    QTabBar::tab:selected, QTabBar::tab:last:selected {
        border-color: #e76500;
        font-style: bold;
        border-width: 2px;
        border-style: inset;
        color: #ffffff;
        padding-left: 8px;
        padding-right: 8px;
        padding-bottom: 0px;
        margin-right: 2px;
        background-color: #333;
    }
    QPushButton{
        margin-top: 3px;
        margin-bottom: 3px;
        margin-left: 8px;
        margin-right: 8px;
        border-style: outset;
        border-top-color: #e18133;
        border-right-color: #e67f2f;
        border-left-color: #e67f2f;
        border-bottom-color: #e67e22;
        border-radius: 8px;
        border-width: 3px;
        padding-top: 4px;
        padding-bottom: 4px;
        font: 14pt bold;
        color: #dedede;
        background-color: #111;
    }
    QPushButton:hover{
        border-style: inset;
        border-width: 3px;
        color: #efefefef;
        background-color: #000;
    }
    QPushButton:pressed{
        border-style: inset;
        border-bottom-width: 3px;
        color: #ffffff;
        background-color: #112;
    }
    QPushButton:disabled{
        border-style: solid;
        border-bottom-width: 1px;
        border-style: solid;
        color: #bbb;
        padding-bottom: 1px;
        background-color: #444;
    }
    QToolButton {
        font-size: 10pt;
        border-style: outset;
        border-color: #e67e22;
        border-width: 3px;
        border-radius: 8px;
        color: #FFF;
        padding: 5px;
        padding-top: 3px;
        background-color: #112;
    }
    QToolButton:hover {
        border-style: inset;
        margin-bottom: 1px;
        padding: 4px;
        background-color: #000;
    }
    QRadioButton {
        background-color: transparent;
        font-size: 12pt;
        border: transparent;
        padding-top: 3px;
        margin-top: 3px;
        margin-bottom: 3px;
        padding-bottom: 3px;
        color: #fff;
    }
    QRadioButton::indicator::unchecked {
        border: 2px inset #f73;
        border-radius: 6px;
        background-color:  #323232;
        width: 13px;
        height: 13px;
    }
    QRadioButton::indicator::unchecked:hover {
        border-radius: 5px;
        background-color:  #323232;
        width: 13px;
        height: 13px;
    }
    QRadioButton::indicator::checked {
        border: 2px inset #d73;
        border-radius: 5px;
        background-color: #f00;
        width: 13px;
        height: 13px;
    }
    """
    """
    QCheckBox {
        color: #fff;
        padding: 4px;
        font-size: 12pt;
        border-color: transparent;
    }
    QCheckBox:disabled {
        color: #808086;
        padding: 6px;
    }
    QCheckBox::indicator:checked {
        height: 13px;
        width: 13px;
        border-style:solid;
        border-width: 2px;
        border-color: #e67e22;
        color: #a9b7c6;
        background-color: #d63d12;
    }
    QCheckBox::indicator:unchecked {
        height: 11px;
        width: 11px;
        border-style:solid;
        border-width: 2px;
        border-color: #e67e22;
        color: #a9b7c6;
        background-color: transparent;
    }"""
    """
    QTreeWidget {
        border: 1px solid #1a1a6a;
        background-color: #5a5a5a;
        font: 8pt;
        color: #eeeeee;
    }
    QTreeWidget::item::selected {
        color: #FFFFFF;
        font-size: 9pt;
        border-color: transparent;
    }
    QTreeWidget::item::hover {
        font-size: 9pt;
        color: #CFF8DC;
        border-color: transparent;
    }
    QTreeWidget::indicator::checked {
        background-color: #ffffff;
        border: 1px solid #536D79;
    }
    QTreeWidget::indicator::unchecked {
        background-color: #ffffff;
        border: 1px solid #536D79;
    }
    QTreeWidget QHeaderView::section {
        background-color: black;
        color: white;
    }
    QTreeWidget QHeaderView::section {
        border: 1px solid #e57e22;
        background-color: black;
        color: white;
    }
    QTreeWidget QProgressBar {
        background-color: #7a7a7a;
        border: 1px solid black;
        border-radius: 3px;
        margin-left: 2px;
        margin-right: 2px;
        text-align: center;
    }
    QTreeWidget QProgressBar::chunk {
        background-color: #3ae1de;
        margin-left: .5px;
        margin-right: .5px;
        margin-top: 1px;
        margin-bottom: 1px;
        border-radius: 3px;
        width: 16px;
    }
    """
    """
    QTableWidget {
        background-color: #5a5a5a;
        color: #ddd;
        border-color: #e57e22;
        border-width: 2px;
        border-style: ridge;
        font-size: 12pt;
        selection-background-color: #3a3a3a;
        selection-color: #FFF;
        gridline-color: #ac4a02;
    }
    QTableWidget QTableCornerButton::section{
        background-color: black;
        border: 2px brown solid;
    }
    QTableWidget QHeaderView {
        border: 1px solid #e57e22;
        font-size: 12pt;
        background-color: black;
        color: white;
    }
    QTableWidget QHeaderView::section {
        border: 1px solid #e57e22;
        background-color: black;
        color: white;
    }
    """
    """
    QMenu{
        background-color:#000000;
        padding-top: 2px;
        padding-bottom: 2px;
    }
    QMenuBar {
        font-size: 10pt;
        background:rgb(30, 30, 30);
        color: #FFFFFF;
        margin-bottom: 1px;
        padding: 3px;
    }
    QMenuBar::item {
        spacing: 3px;
        padding: 1px 4px;
        padding-left: 6px;
        padding-right: 6px;
        padding-top: 3px;
        padding-bottom: 3px;
        background: transparent;
    }
    QMenuBar::item:selected {
        border-style: solid;
        border-top-color: transparent;
        border-right-color: transparent;
        border-left-color: transparent;
        border-bottom-color: #e67e22;
        border-bottom-width: 1px;
        border-style: solid;
        color: #FFFFFF;
        padding-bottom: 0px;
        background-color: #000000;
    }
    QMenu::item:selected {
        border-style: solid;
        border-top-color: transparent;
        border-right-color: transparent;
        border-left-color: #e67e22;
        border-bottom-color: transparent;
        border-left-width: 2px;
        color: #FFFFFF;
        padding-left:15px;
        padding-top:4px;
        padding-bottom:4px;
        padding-right:7px;
        background-color:#000000;
    }
    QMenu::item {
        border-style: solid;
        font-size: 10px;
        border-top-color: transparent;
        border-right-color: transparent;
        border-left-color: transparent;
        border-bottom-color: transparent;
        border-bottom-width: 1px;
        border-style: solid;
        color: #a9b7c6;
        padding-left:17px;
        padding-top:4px;
        padding-bottom:4px;
        padding-right:7px;
        background-color:#000000;
    }"""
    """
    QDialog {
        background-color:#000000;
    }
    QLineEdit {
        padding: 4px;
        margin-top: 3px;
        margin-bottom: 3px;
        font: 10pt;
    }
    QLabel {
        padding: 2px;
        font-weight: bold;
    }
    """
)

infoLineEdit = """
    QLineEdit {
        background-color: transparent;
        color: #FFF;
        border-radius: 0px;
        border-width: 2px;
        font-size: 11pt;
        padding: 0px;
        margin: 0px;
        border-left-color: transparent;
        border-right-color: transparent;
        border-top-color: transparent;
    }

"""

pushButtonEdit = """
    QPushButton {
        padding: 3px;
        margin: 0px;
        font-size: 9pt;
    }
"""



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Widgets and procedures for the "Torrent Editor" tab."""

import os
from pathlib import Path

import pyben
from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtWidgets import (QFileDialog, QHBoxLayout, QLabel, QLineEdit,
                             QPushButton, QTableWidget, QTableWidgetItem,
                             QToolButton, QVBoxLayout, QWidget)

# from torrentfileQt.qss import (labelSheet, lineEditSheet, pushButtonSheet,
#                                tableSheet, toolButtonSheet)


class EditorWidget(QWidget):
    """Main widget for the torrent editor tab."""

    def __init__(self, parent=None):
        """Construct editor tab widget.

        Args:
            parent (`QWidget`): parent widget of this widge.
        """
        super().__init__(parent=parent)
        self.window = parent.window
        self.layout = QVBoxLayout()
        self.line = LineEdit(parent=self)
        self.button = Button("Save", parent=self)
        self.fileButton = FileButton(parent=self)
        self.label = QLabel("Torrent File:", parent=self)
        # self.label.setStyleSheet(labelSheet)
        self.table = Table(parent=self)
        self.hlayout = QHBoxLayout()
        self.hlayout.addWidget(self.label)
        self.hlayout.addWidget(self.line)
        self.hlayout.addWidget(self.fileButton)
        self.layout.addLayout(self.hlayout)
        self.layout.addWidget(self.table)
        self.layout.addWidget(self.button)
        self.setLayout(self.layout)


class LineEdit(QLineEdit):
    """Line edit class from QLineEdit."""

    def __init__(self, parent=None):
        """Constructor for the LineEdit on torrent editor tab."""
        super().__init__(parent=parent)
        # self.setStyleSheet(lineEditSheet)
        self.setDisabled(True)
        self.widget = parent
        self.window = parent.window


class Button(QPushButton):
    """Button Widget for saving results to .torrent file."""

    def __init__(self, text, parent=None):
        """Constructor for the save button on torrent editor tab."""
        super().__init__(text, parent=parent)
        # self.setStyleSheet(pushButtonSheet)
        self.widget = parent
        self.pressed.connect(self.save)

    def save(self):
        """Save method for writing edit results to .torrent file."""
        table = self.widget.table
        text = self.widget.line.text()
        meta = table.original
        info = meta["info"]
        for row in range(table.rowCount()):
            label = table.item(row, 0).text()
            value = table.item(row, 1).text()
            if label in ["piece length", "private", "creation date"]:
                value = int(value)
            if label in meta and meta[label] != value:
                meta[label] = value
            elif label in info and info[label] != value:
                info[label] = value
        pyben.dump(meta, text)


class FileButton(QToolButton):
    """Tool Button for selecting a .torrent file to edit."""

    def __init__(self, parent=None):
        """Constructor for the FileDialog button on Torrent Editor tab."""
        super().__init__(parent=parent)
        self.widget = parent
        self.setText("...")
        self.window = parent.window
        # self.setStyleSheet(toolButtonSheet)
        self.pressed.connect(self.browse)

    def browse(self, path=None):
        """Browse method for finding the .torrent file user wishes to edit."""
        if not path:  # pragma: no coverage
            path = QFileDialog.getOpenFileName(
                directory=str(Path().home),
                caption="Select Torrent File",
                filter="*.torrent",
            )[0]
        self.widget.table.clear()
        self.widget.line.setText(path)
        self.widget.table.handleTorrent.emit(path)


class Table(QTableWidget):
    """Table widget for displaying editable information from .torrent file."""

    handleTorrent = pyqtSignal([str])

    def __init__(self, parent=None):
        """Constructor for the Table Widget on torrent editor tab."""
        super().__init__(parent=parent)
        # self.setStyleSheet(tableSheet)
        self.info = {}
        self.window = parent.window
        self.original = None
        self.setColumnCount(2)
        self.setRowCount(0)
        header = self.horizontalHeader()
        header.setStretchLastSection(True)
        vheader = self.verticalHeader()
        vheader.setSectionResizeMode(vheader.ResizeMode.Stretch)
        self.setHorizontalHeaderLabels(["Label", "Value"])
        self.handleTorrent.connect(self.export_data)

    def clear(self):
        """Remove any data previously added to table."""
        self.info = {}
        self.setRowCount(0)
        super().clear()

    def export_data(self, path):
        """Slot for the handleTorrent signal."""
        if not os.path.exists(path):  # pragma: no cover
            return
        self.original = pyben.load(path)
        self.flatten_data(self.original)
        counter = 0
        for k, v in self.info.items():
            self.window.app.processEvents()
            self.setRowCount(self.rowCount() + 1)
            item = QTableWidgetItem(0)
            item.setText(str(k))
            item.setFlags(item.flags() ^ Qt.ItemFlag.ItemIsEditable)
            self.setItem(counter, 0, item)
            item2 = QTableWidgetItem(0)
            item2.setText(str(v))
            self.setItem(counter, 1, item2)
            counter += 1

    def flatten_data(self, data):
        """Flatten the meta dictionary found in the selected .torrent file."""
        for k, v in data.items():
            if k in [
                "source",
                "private",
                "announce",
                "name",
                "piece length",
                "comment",
                "creation date",
                "created by",
                "announce list",
            ]:
                self.info[k] = v
            elif k == "info":
                self.flatten_data(v)



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Widgets and procedures for the "Torrent Info" tab."""

import math
import os
from datetime import datetime
from pathlib import Path

import pyben
from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtGui import QIcon
from PyQt6.QtWidgets import (QFileDialog, QGridLayout, QLabel, QLineEdit,
                             QPushButton, QTreeWidget, QTreeWidgetItem,
                             QWidget)

from torrentfileQt.qss import infoLineEdit


class TreeWidget(QTreeWidget):
    """Tree view of the directory structure cataloged in .torrent file.

    Args:
        parent (`widget`, default=`None`): The widget containing this widget.
    """

    itemReady = pyqtSignal([str, int])

    def __init__(self, parent=None):
        """Constructor for tree widget."""
        super().__init__(parent=parent)
        self.window = parent.window
        self.widget = parent
        self.root = self.invisibleRootItem()
        header = self.header()
        header.setSectionResizeMode(0, header.ResizeMode.ResizeToContents)
        header.setSectionResizeMode(1, header.ResizeMode.ResizeToContents)
        self.root.setChildIndicatorPolicy(
            self.root.ChildIndicatorPolicy.ShowIndicator
        )
        self.setIndentation(8)
        self.setEditTriggers(self.EditTrigger.NoEditTriggers)
        self.setHeaderHidden(True)
        self.setItemsExpandable(True)
        self.setColumnCount(2)
        self.itemtree = {"widget": self.root}
        self.itemReady.connect(self.apply_value)

    def apply_value(self, text, length):
        """Add path partials and file lengths to tree branches."""
        partials = Path(text).parts
        tree = self.itemtree
        for i, partial in enumerate(partials):
            if partial not in tree:
                item = TreeItem(0)
                if i + 1 == len(partials):
                    iconpath = "./assets/file.png"
                    item.setLength(length)
                else:
                    iconpath = "./assets/folder.png"
                icon = QIcon(iconpath)
                item.setIcon(0, icon)
                item.setText(1, partial)
                tree["widget"].addChild(item)
                item.setExpanded(True)
                tree[partial] = {"widget": item}
            tree = tree[partial]


class TreeItem(QTreeWidgetItem):
    """Item widget for tree leaves and branches."""

    def __init__(self, group):
        """Constructor for tree item widget."""
        super().__init__(group)
        policy = self.ChildIndicatorPolicy.DontShowIndicatorWhenChildless
        self.setChildIndicatorPolicy(policy)

    def setLength(self, length):
        """Set length leaf for tree branches."""
        child = TreeItem(0)
        icon = QIcon("./assets/scale.png")
        child.setIcon(0, icon)
        child.setText(1, f"Size: {length} (bytes)")
        self.setExpanded(True)
        self.addChild(child)


class InfoWidget(QWidget):
    """Main parent widget for the Torrent Info tab."""

    def __init__(self, parent=None):
        """Construct and organize Torrent info tab."""
        super().__init__(parent=parent)
        self.window = parent.window
        self.layout = QGridLayout()
        self.setLayout(self.layout)

        # Labels
        self.pathLabel = Label("Path: ", parent=self)
        self.nameLabel = Label("Name: ", parent=self)
        self.pieceLengthLabel = Label("Piece Length: ", parent=self)
        self.sizeLabel = Label("Total Size: ", parent=self)
        self.totalPiecesLabel = Label("Total Pieces: ", parent=self)
        self.trackerLabel = Label("Tracker: ", parent=self)
        self.privateLabel = Label("Private: ", parent=self)
        self.sourceLabel = Label("Source: ", parent=self)
        self.commentLabel = Label("Comment: ", parent=self)
        self.metaVersionLabel = Label("Meta-Version:", parent=self)
        self.dateCreatedLabel = Label("Creation Date: ", parent=self)
        self.createdByLabel = Label("Created By: ", parent=self)
        self.contentsLabel = Label("Contents: ", parent=self)
        self.contentsTree = TreeWidget(parent=self)
        self.sourceEdit = InfoLineEdit(parent=self)
        self.metaVersionEdit = InfoLineEdit(parent=self)
        self.pathEdit = InfoLineEdit(parent=self)
        self.nameEdit = InfoLineEdit(parent=self)
        self.pieceLengthEdit = InfoLineEdit(parent=self)
        self.sizeEdit = InfoLineEdit(parent=self)
        self.privateEdit = InfoLineEdit(parent=self)
        self.commentEdit = InfoLineEdit(parent=self)
        self.trackerEdit = InfoLineEdit(parent=self)
        self.totalPiecesEdit = InfoLineEdit(parent=self)
        self.dateCreatedEdit = InfoLineEdit(parent=self)
        self.createdByEdit = InfoLineEdit(parent=self)
        self.layout.addWidget(self.pathLabel, 0, 0, 1, 1)
        self.layout.addWidget(self.pathEdit, 0, 1, 1, -1)
        self.layout.addWidget(self.nameLabel, 1, 0, 1, 1)
        self.layout.addWidget(self.nameEdit, 1, 1, 1, -1)
        self.layout.addWidget(self.sizeLabel, 2, 0, 1, 1)
        self.layout.addWidget(self.sizeEdit, 2, 1, 1, -1)
        self.layout.addWidget(self.pieceLengthLabel, 3, 0, 1, 1)
        self.layout.addWidget(self.pieceLengthEdit, 3, 1, 1, -1)
        self.layout.addWidget(self.totalPiecesLabel, 4, 0, 1, 1)
        self.layout.addWidget(self.totalPiecesEdit, 4, 1, 1, -1)
        self.layout.addWidget(self.trackerLabel, 5, 0, 1, 1)
        self.layout.addWidget(self.trackerEdit, 5, 1, 1, -1)
        self.layout.addWidget(self.metaVersionLabel, 6, 0, 1, 1)
        self.layout.addWidget(self.metaVersionEdit, 6, 1, 1, -1)
        self.layout.addWidget(self.privateLabel, 7, 0, 1, 1)
        self.layout.addWidget(self.privateEdit, 7, 1, 1, -1)
        self.layout.addWidget(self.sourceLabel, 8, 0, 1, 1)
        self.layout.addWidget(self.sourceEdit, 8, 1, 1, -1)
        self.layout.addWidget(self.commentLabel, 9, 0, 1, 1)
        self.layout.addWidget(self.commentEdit, 9, 1, 1, -1)
        self.layout.addWidget(self.createdByLabel, 10, 0, 1, 1)
        self.layout.addWidget(self.createdByEdit, 10, 1, 1, -1)
        self.layout.addWidget(self.dateCreatedLabel, 11, 0, 1, 1)
        self.layout.addWidget(self.dateCreatedEdit, 11, 1, 1, -1)
        self.layout.addWidget(self.contentsLabel, 12, 0, 1, 1)
        self.layout.addWidget(self.contentsTree, 12, 1, 4, -1)
        self.selectButton = SelectButton("Select Torrent", parent=self)
        self.layout.addWidget(self.selectButton, 17, 0, -1, -1)
        self.pathLabel.setObjectName("pathLabel")
        self.pathEdit.setObjectName("pathEdit")
        self.nameLabel.setObjectName("nameLabel")
        self.nameEdit.setObjectName("nameEdit")
        self.sizeLabel.setObjectName("sizeLabel")
        self.sizeEdit.setObjectName("sizeEdit")
        self.pieceLengthLabel.setObjectName("pieceLengthLabel")
        self.pieceLengthEdit.setObjectName("pieceLengthEdit")
        self.totalPiecesLabel.setObjectName("totalPiecesLabel")
        self.totalPiecesEdit.setObjectName("totalPiecesEdit")
        self.trackerLabel.setObjectName("trackerLabel")
        self.trackerEdit.setObjectName("trackerEdit")
        self.metaVersionLabel.setObjectName("metaVersionLabel")
        self.metaVersionEdit.setObjectName("metaVersionEdit")
        self.privateLabel.setObjectName("privateLabel")
        self.privateEdit.setObjectName("privateEdit")
        self.sourceLabel.setObjectName("sourceLabel")
        self.sourceEdit.setObjectName("sourceEdit")
        self.commentLabel.setObjectName("commentLabel")
        self.commentEdit.setObjectName("commentEdit")
        self.createdByLabel.setObjectName("createdByLabel")
        self.createdByEdit.setObjectName("createdByEdit")
        self.dateCreatedLabel.setObjectName("dateCreatedLabel")
        self.dateCreatedEdit.setObjectName("dateCreatedEdit")
        self.contentsLabel.setObjectName("contentsLabel")
        self.contentsTree.setObjectName("contentsTree")

    def fill(self, **kws):
        """Fill all child widgets with collected information.

        Args:
            kws (`dict`): key, value dictionary with keys as field labels.
        """
        self.pathEdit.setText(kws["path"])
        self.nameEdit.setText(kws["name"])
        self.commentEdit.setText(kws["comment"])
        self.trackerEdit.setText("; ".join(kws["announce"]))
        self.sourceEdit.setText(kws["source"])
        self.dateCreatedEdit.setText(str(kws["creation_date"]))
        self.createdByEdit.setText(kws["created_by"])
        self.privateEdit.setText(kws["private"])
        self.metaVersionEdit.setText(str(kws["meta version"]))

        piece_length = kws["piece_length"]
        plength_str = (
            denom(piece_length) + " / (" + pretty_int(piece_length) + ")"
        )
        self.pieceLengthEdit.setText(plength_str)
        size = denom(kws["length"]) + " / (" + pretty_int(kws["length"]) + ")"
        self.sizeEdit.setText(size)
        total_pieces = math.ceil(kws["length"] / kws["piece_length"])
        self.totalPiecesEdit.setText(str(total_pieces))
        for path, size in kws["contents"].items():
            self.contentsTree.itemReady.emit(path, size)
        for widg in [
            self.pathEdit,
            self.nameEdit,
            self.trackerEdit,
            self.privateEdit,
            self.pieceLengthEdit,
            self.sizeEdit,
            self.totalPiecesEdit,
        ]:
            widg.setCursorPosition(0)


class SelectButton(QPushButton):
    """Button for choosing the torrent file."""

    def __init__(self, text, parent=None):
        """Constructor for select button."""
        super().__init__(text, parent=parent)
        # self.setStyleSheet(pushButtonSheet)
        self.pressed.connect(self.selectTorrent)
        self.setCursor(Qt.CursorShape.PointingHandCursor)

    def selectTorrent(self, path=None):
        """Collect torrent information and send to the screen for display."""
        caption = "Select '.torrent' file"
        if not path:  # pragma: no cover
            path, _ = QFileDialog.getOpenFileName(
                parent=self, caption=caption, filter="*.torrent"
            )
        if not path:  # pragma: no cover
            return
        meta = pyben.load(path)
        info = meta["info"]
        keywords = {}
        keywords["path"] = path
        keywords["piece_length"] = info["piece length"]
        if "meta version" not in info:
            keywords["meta version"] = 1
        elif "pieces" in info:
            keywords["meta version"] = 3
        else:
            keywords["meta version"] = 2
        if "created by" in meta:
            keywords["created_by"] = meta["created by"]
        else:
            keywords["created_by"] = ""  # pragma: no cover
        for kw in ["name", "length", "comment", "source"]:
            if kw in info:
                keywords[kw] = info[kw]
            else:
                keywords[kw] = ""
        if "announce list" in info:
            keywords["announce"] = info["announce list"] + [meta["announce"]]
        else:  # pragma: no cover
            keywords["announce"] = [meta["announce"]]
        size = 0
        if "files" in info:
            contents = {}
            for entry in info["files"]:
                contents[os.path.join(info["name"], *entry["path"])] = entry[
                    "length"
                ]
                size += entry["length"]
            keywords["contents"] = contents
        elif "file tree" in info:
            contents = {}
            for k, v in parse_filetree(info["file tree"]).items():
                contents[os.path.join(info["name"], k)] = v
                size += v
            keywords["contents"] = contents
        else:  # pragma: no cover
            keywords["contents"] = {info["name"]: info["length"]}
            size = info["length"]
        keywords["length"] = size
        if "creation date" in meta:
            date = datetime.fromtimestamp(meta["creation date"])
            text = date.strftime("%B %d, %Y %H:%M")
            keywords["creation_date"] = text
        else:
            keywords["creation_date"] = ""  # pragma: no cover
        if "private" in info:
            keywords["private"] = "True"
        else:  # pragma: no cover
            keywords["private"] = "False"
        self.parent().fill(**keywords)


class Label(QLabel):
    """Label Identifier for Window Widgets."""

    def __init__(self, text, parent=None):
        """Constructor for Label Widget."""
        super().__init__(text, parent=parent)
        font = self.font()
        font.setBold(True)
        font.setPointSize(11)
        self.setFont(font)


class InfoLineEdit(QLineEdit):
    """Line Edit Widget."""

    def __init__(self, parent=None):
        """Constructor for line edit widget."""
        super().__init__(parent=parent)
        self.setReadOnly(True)
        self.setStyleSheet(infoLineEdit)
        self.setDragEnabled(True)


def denom(num):
    """Determine appropriate denomination for size of file."""
    txt = str(num)
    if int(num) < 1000:
        return txt
    if 1000 <= num <= 999999:
        return "".join([txt[:-3], ".", txt[-3], "KB"])
    if 1_000_000 <= num < 1_000_000_000:
        return "".join([txt[:-6], ".", txt[-6], "MB"])
    if num >= 1_000_000_000:
        return "".join([txt[:-9], ".", txt[-9], "GB"])
    return str(num)  # pragma: no cover


def pretty_int(num):
    """Format integer."""
    text, seq = str(num), []
    digits, count = len(text) - 1, 0
    while digits >= 0:
        if count == 3:
            seq.insert(0, ",")
            count = 0
        seq.insert(0, text[digits])
        count += 1
        digits -= 1
    return "".join(seq) + " Bytes"


def parse_filetree(filetree):
    """Iterate through dictionary to create pathstrings."""
    paths = {}
    for key, value in filetree.items():
        if "" in value:
            paths[key] = value[""]["length"]
        else:
            out = parse_filetree(value)
            for k, v in out.items():
                paths[os.path.join(key, k)] = v
    return paths



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Module for the menu bar."""

import os
import webbrowser
from pathlib import Path

import pyben
from PyQt6.QtGui import QAction
from PyQt6.QtWidgets import QFileDialog, QMenu, QMenuBar

# from .qss import menuSheet


class Menu(QMenu):
    """Menu item for MenuBar widget."""

    def __init__(self, text, parent=None):
        """Constructor for Menu Widget."""
        super().__init__(text, parent=parent)
        self.menubar = parent
        self.txt = text
        font = self.font()
        self.setObjectName(text)
        font.setPointSize(12)
        self.setFont(font)


class MenuBar(QMenuBar):
    """Main menu bar for top level menu of program."""

    def __init__(self, parent=None):
        """Constructor for top level widgets."""
        super().__init__(parent=parent)
        self.window = parent
        # self.setStyleSheet(menuSheet)
        self.file_menu = Menu("File")
        self.help_menu = Menu("Help")
        self.addMenu(self.file_menu)
        self.addMenu(self.help_menu)
        self.actionExport = QAction(self.window)
        self.actionExit = QAction(self.window)
        self.actionAbout = QAction(self.window)
        self.actionDocs = QAction(self.window)
        self.actionRepo = QAction(self.window)
        self.actionRepo.setText("Github Repository")
        self.actionExit.setText("Exit")
        self.actionAbout.setText("About")
        self.actionDocs.setText("Documentation")
        self.actionExport.setText("Save to")
        self.file_menu.addAction(self.actionExit)
        self.file_menu.addAction(self.actionExport)
        self.help_menu.addAction(self.actionAbout)
        self.help_menu.addAction(self.actionDocs)
        self.help_menu.addAction(self.actionRepo)
        self.actionExit.triggered.connect(self.exit_app)
        self.actionAbout.triggered.connect(self.about_qt)
        self.actionExport.triggered.connect(self.export)
        self.actionDocs.triggered.connect(documentation)
        self.actionRepo.triggered.connect(repository)
        self.actionDocs.setObjectName("actionDocs")
        self.actionExit.setObjectName("actionExit")
        self.actionAbout.setObjectName("actionAbout")
        self.actionExport.setObjectName("actionExport")

    def export(self, path=None):
        """Export data from info widget to file."""
        home = str(Path.home())
        if path:
            filename = path
        else:  # pragma: no cover
            filename = QFileDialog.getSaveFileName(
                caption="Save location:", directory=home
            )
        if os.path.exists(os.path.dirname(filename)):
            widget = self.window.central.infoWidget
            path = widget.pathEdit.text()
            if os.path.exists(path):
                data = pyben.load(path)
                with open(filename, "wt") as fd:
                    fd.write(str(data))

    def about_qt(self):
        """Open the about qt menu."""
        self.window.app.aboutQt()  # pragma: nocover

    def exit_app(self):
        """Close application."""
        self.parent().app.quit()  # pragma: nocover


def documentation():  # pragma: no cover
    """Open webbrowser to TorrentFileQt documentation."""
    webbrowser.open_new_tab("https://alexpdev.github.io/torrentfile")


def repository():  # pragma: no cover
    """Open webbrowser to GitHub Repo."""
    webbrowser.open_new_tab("https://github.com/alexpdev/torrentfileQt")



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Module for the Check Tab Widget."""

import logging
import os
import re
from collections.abc import Sequence
from pathlib import Path

from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtGui import QIcon
from PyQt6.QtWidgets import (QFileDialog, QFormLayout, QHBoxLayout, QLabel,
                             QLineEdit, QPlainTextEdit, QProgressBar,
                             QPushButton, QSplitter, QToolButton, QTreeWidget,
                             QTreeWidgetItem, QVBoxLayout, QWidget)
from torrentfile.progress import CheckerClass

# from torrentfileQt.qss import (headerSheet, labelSheet, lineEditSheet,
#                                logTextEditSheet, pushButtonSheet,
#                                toolButtonSheet, treeSheet)


class CheckWidget(QWidget):
    """Check tab widget for QMainWindow."""

    def __init__(self, parent=None):
        """Constructor for check tab."""
        super().__init__(parent=parent)
        self.window = parent.window
        self.vlayout = QVBoxLayout()
        self.layout = QFormLayout()
        self.setLayout(self.vlayout)
        self.vlayout.addLayout(self.layout)
        self.splitter = QSplitter(parent=self)
        self.vlayout.addWidget(self.splitter)

        self.hlayout1 = QHBoxLayout()
        self.hlayout2 = QHBoxLayout()

        self.fileLabel = Label("Torrent File", parent=self)
        self.fileLabel.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.fileInput = LineEdit(parent=self)
        self.browseButton1 = BrowseTorrents(parent=self)

        self.searchLabel = Label("Search Path", parent=self)
        self.searchLabel.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.searchInput = LineEdit(parent=self)
        self.browseButton2 = BrowseFolders.create(
            parent=self, text="Folder", mode=0
        )
        self.browseButton3 = BrowseFolders.create(
            parent=self, text="File", mode=1
        )
        self.checkButton = ReCheckButton("Check", parent=self)

        self.hlayout1.addWidget(self.fileInput)
        self.hlayout1.addWidget(self.browseButton1)
        self.hlayout2.addWidget(self.searchInput)
        self.hlayout2.addWidget(self.browseButton2)
        self.hlayout2.addWidget(self.browseButton3)

        labelRole = QFormLayout.ItemRole.LabelRole
        fieldRole = QFormLayout.ItemRole.FieldRole
        self.layout.setWidget(1, labelRole, self.fileLabel)
        self.layout.setLayout(1, fieldRole, self.hlayout1)
        self.layout.setWidget(2, labelRole, self.searchLabel)
        self.layout.setLayout(2, fieldRole, self.hlayout2)
        self.textEdit = LogTextEdit(parent=self)
        self.treeWidget = TreeWidget(parent=self)
        self.splitter.addWidget(self.treeWidget)
        self.splitter.addWidget(self.textEdit)
        self.splitter.setOrientation(Qt.Orientation.Vertical)
        self.vlayout.addWidget(self.checkButton)

        self.layout.setObjectName("CheckWidget_layout")
        self.hlayout1.setObjectName("CheckWidget_hlayout1")
        self.hlayout2.setObjectName("CheckWidget_hlayout2")
        self.browseButton2.setObjectName("CheckWidget_browseButton2")
        self.browseButton1.setObjectName("CheckWidget_browseButton1")
        self.fileLabel.setObjectName("CheckWidget_fileLabel")
        self.searchLabel.setObjectName("CheckWidget_searchLabel")
        self.fileInput.setObjectName("CheckWidget_fileInput")
        self.searchInput.setObjectName("CheckWidget_searchInput")


class ReCheckButton(QPushButton):
    """Button Widget for validating torrent files against downloaded contents.

    Args:
        text (`str`): The text displayed on the button itself.
        parent (`QWidget`, default=None): This widgets parent widget.
    """

    process = None

    def __init__(self, text, parent=None):
        """Construct the CheckButton Widget."""
        super().__init__(text, parent=parent)
        self.widget = parent
        self.window = parent.window
        self.pressed.connect(self.submit)
        self.setCursor(Qt.CursorShape.PointingHandCursor)
        # self.setStyleSheet(pushButtonSheet)

    def submit(self):
        """Submit data to piece hasher."""
        tree = self.widget.treeWidget
        tree.clear()
        textEdit = self.widget.textEdit
        textEdit.clear()
        searchInput = self.widget.searchInput
        fileInput = self.widget.fileInput
        metafile = fileInput.text()
        content = searchInput.text()
        if os.path.exists(metafile):
            CheckerClass.register_callback(textEdit.callback)
            logging.debug("Registering Callback, setting root")
            tree.reChecking.emit(metafile, content)


class BrowseTorrents(QToolButton):
    """BrowseButton ToolButton for activating filebrowser.

    Args:
        parent (`widget`): Parent widget.
    """

    def __init__(self, parent=None):
        """Construct Toolbar Button for selecting .torrentfile to check."""
        super().__init__(parent=parent)
        self.setText("...")
        self.window = parent
        # self.setStyleSheet(toolButtonSheet)
        self.setCursor(Qt.CursorShape.PointingHandCursor)
        self.pressed.connect(self.browse)

    def browse(self, path=None):
        """Browse action performed when user presses button.

        Returns:
            path (`str`): Path to file or folder to include in torrent.
        """
        caption = "Choose .torrent file."
        if not path:  # pragma: no cover
            path = QFileDialog.getOpenFileName(
                parent=self, caption=caption, filter="*.torrent"
            )
        if path:
            if isinstance(path, Sequence):
                path = path[0]
            path = os.path.normpath(path)
            self.parent().fileInput.clear()
            self.parent().fileInput.setText(path)


class BrowseFolders(QToolButton):
    """Browse Folders ToolButton for activating filedialog.

    Args:
        parent (`QWidget`, default=None): Widget this widget is the child of.
    """

    modes = {
        0: {
            "func": QFileDialog.getExistingDirectory,
            "caption": "Select Contents Folder...",
            "directory": str(Path.home()),
        },
        1: {
            "func": QFileDialog.getOpenFileName,
            "caption": "Select Contents File...",
            "directory": str(Path.home()),
        },
    }

    def __init__(self, parent=None):
        """Construct a BrowseFolders Button Widget."""
        super().__init__(parent=parent)
        self.window = parent
        # self.setStyleSheet(toolButtonSheet)
        self.setCursor(Qt.CursorShape.PointingHandCursor)
        self.mode = None
        self.pressed.connect(self.browse)

    @classmethod
    def create(cls, parent=None, text=None, mode=None):
        """Create new instance of button with mode."""
        btn = cls(parent=parent)
        btn.setText(text)
        btn.mode = mode
        return btn

    def browse(self, path=None):
        """Browse Action performed when user presses button.

        Returns:
            `str`: Path to file or folder to include in torrent.
        """
        if not path:  # pragma: no cover
            mode = self.modes[self.mode]
            path = mode["func"](
                directory=mode["directory"],
                parent=self,
                caption=mode["caption"],
            )
        if path:
            path = os.path.normpath(path)
            self.parent().searchInput.clear()
            self.parent().searchInput.setText(path)


class LineEdit(QLineEdit):
    """Line edit widget."""

    def __init__(self, parent=None):
        """Constructor for line edit widget."""
        super().__init__(parent=parent)
        # self.setStyleSheet(lineEditSheet)


class LogTextEdit(QPlainTextEdit):
    """Text Edit widget for check tab."""

    def __init__(self, parent=None):
        """Constructor for LogTextEdit."""
        super().__init__(parent=parent)
        self._parent = parent
        self.setBackgroundVisible(True)
        font = self.font()
        font.setFamily("Consolas")
        font.setBold(True)
        self.setFont(font)
        # self.setStyleSheet(logTextEditSheet)

    def clear_data(self):
        """Remove any text."""
        self.clear()

    def callback(self, msg):
        """Callback function for CheckerClass."""
        self.insertPlainText(msg)
        self.insertPlainText("\n")


class Label(QLabel):
    """Label Identifier for Window Widgets.

    Subclass: QLabel
    """

    def __init__(self, text, parent=None):
        """Constructor for Label."""
        super().__init__(text, parent=parent)
        font = self.font()
        # self.setStyleSheet(labelSheet)
        font.setBold(True)
        font.setPointSize(12)
        self.setFont(font)


class TreePieceItem(QTreeWidgetItem):
    """Item Widgets that are leafs to Tree Widget branches."""

    def __init__(self, type=0, tree=None):
        """Constructor for tree widget items."""
        super().__init__(type=type)
        policy = self.ChildIndicatorPolicy.DontShowIndicatorWhenChildless
        self.setChildIndicatorPolicy(policy)
        self.tree = tree
        self.window = tree.window
        self.counted = self.value = 0
        self.progbar = None

    @property
    def total(self):
        """Returns current value of progress bar."""
        return self.progbar.total

    @property
    def left(self):
        """Remaining amount of data left to check."""
        return self.progbar.total - self.counted

    def addProgress(self, value):
        """Increase progress bar value."""
        if self.counted + value > self.total:
            consumed = self.total - self.value  # pragma: no cover
        else:
            consumed = value
        self.value += consumed
        self.counted += consumed
        self.progbar.valueChanged.emit(consumed)
        self.window.app.processEvents()
        return consumed

    def count(self, value):
        """Increase count without increasing value."""
        if self.counted + value > self.total:
            consumed = self.total - self.value
            self.counted += consumed
            return consumed
        self.counted += value
        return value


class ProgressBar(QProgressBar):
    """Progress Bar Widget."""

    valueChanged = pyqtSignal([int])

    def __init__(self, parent=None, size=0):
        """Constructor for the progress bar widget."""
        super().__init__(parent=parent)
        self.total = size
        self.setValue(0)
        self.setRange(0, size)
        self.valueChanged.connect(self.addValue)

    def addValue(self, value):
        """Increase value of progressbar."""
        currentvalue = self.value()
        addedVal = currentvalue + value
        self.setValue(addedVal)


class TreeWidget(QTreeWidget):
    """Tree Widget for the `Check` tab.

    Displays percentages for matching files and their progress.

    Args:
        parent(`QWidget`, default=None)
    """

    addPathChild = pyqtSignal([str, int])
    reChecking = pyqtSignal([str, str])
    addValue = pyqtSignal([str, int])
    addCount = pyqtSignal([str, int])

    def __init__(self, parent=None):
        """Constructor for Tree Widget."""
        super().__init__(parent=parent)
        self.window = parent.window
        # self.setStyleSheet(treeSheet + headerSheet)
        self.setColumnCount(3)
        self.setIndentation(10)
        self.item = self.invisibleRootItem()
        self.item.setExpanded(True)
        header = self.header()
        header.setSectionResizeMode(0, header.ResizeMode.ResizeToContents)
        header.setSectionResizeMode(1, header.ResizeMode.ResizeToContents)
        self.setHeaderHidden(True)
        self.itemWidgets = {}
        self.paths = []
        self.total = 0
        self.root = None
        self.piece_length = None
        self.item_tree = {"widget": self.item}
        self.addPathChild.connect(self.add_path_child)
        self.reChecking.connect(self.get_hashes)
        self.addValue.connect(self.setItemValue)
        self.addCount.connect(self.setItemCount)

    def setItemValue(self, path, val):
        """Set child widgets value to val."""
        widget = self.itemWidgets[path]
        widget.addProgress(val)

    def setItemCount(self, path, val):
        """Set child widgets count to val."""
        widget = self.itemWidgets[path]
        widget.count(val)

    def get_hashes(self, metafile, contents):
        """Fill tree widget with contents of torrentfile."""
        phashes = PieceHasher(metafile, contents, self)
        phashes.addTreeWidgets()
        phashes.iter_hashes()

    def clear(self):
        """Remove any objects from Tree Widget."""
        super().clear()
        self.item_tree = {"widget": self.invisibleRootItem()}
        self.itemWidgets = {}
        self.paths = []
        self.root = None

    def add_path_child(self, path, size):
        """Add branch to tree."""
        path = Path(path)
        partials = path.parts
        item, item_tree = None, self.item_tree
        for i, partial in enumerate(partials):
            if partial in item_tree:
                item_tree = item_tree[partial]
                continue
            parent = item_tree["widget"]
            item = TreePieceItem(0, tree=self)
            parent.addChild(item)
            parent.setExpanded(True)
            item_tree[partial] = {"widget": item}
            if i == len(partials) - 1:
                if path.suffix in [".avi", ".mp4", ".mkv", ".mov"]:
                    fileicon = QIcon("./assets/video.png")
                elif path.suffix in [".rar", ".zip", ".gz", ".7z"] or re.match(
                    r"\.r\d+$", path.suffix
                ):
                    fileicon = QIcon("./assets/archive.png")
                else:
                    fileicon = QIcon("./assets/file.png")
                progressBar = ProgressBar(parent=None, size=size)
                self.setItemWidget(item, 2, progressBar)
                item.progbar = progressBar
                self.itemWidgets[str(path)] = item
            else:
                fileicon = QIcon("./assets/folder.png")
            item.setIcon(0, fileicon)
            item.setText(1, partial)
            item_tree = item_tree[partial]
            self.window.app.processEvents()
        self.paths.append(path)


class PieceHasher:
    """Piece Hasher class for iterating through captured torrent pieces."""

    def __init__(self, metafile, content, tree):
        """Constructor for PieceHasher class."""
        self.metafile = metafile
        self.content = content
        self.tree = tree
        self.checker = CheckerClass(metafile, content)
        self.root = os.path.dirname(self.checker.root)
        self.fileinfo = self.checker.fileinfo
        self.pathlist = self.checker.paths
        self.current = 0

    def addTreeWidgets(self):
        """Add tree widgets items to tree widget."""
        for path in self.pathlist:
            if path == self.root:
                relpath = os.path.split(self.root)[-1]  # pragma: no cover
            else:
                relpath = os.path.relpath(path, self.root)
            length = self.fileinfo[path]["length"]
            self.tree.addPathChild.emit(relpath, length)

    def iter_hashes(self):
        """Iterate through hashes and compare to torrentfile hashes."""
        for actual, expected, path, size in self.checker.iter_hashes():
            if self.checker.meta_version == 1:
                while size > 0:
                    if self.current >= len(self.pathlist):
                        break  # pragma: no cover
                    current = self.pathlist[self.current]
                    relpath = os.path.relpath(current, self.root)
                    widget = self.tree.itemWidgets[relpath]
                    if widget.left == 0:
                        self.current += 1
                        continue
                    left, amount = widget.left, None
                    if actual == expected:
                        amount = left if left < size else size
                        self.tree.addValue.emit(relpath, amount)
                    else:
                        amount = left if left < size else size
                        self.tree.addCount.emit(relpath, amount)
                    size -= amount
            else:
                if path == self.root:
                    relpath = os.path.split(self.root)[-1]  # pragma: no cover
                else:
                    relpath = os.path.relpath(path, self.root)
                if actual == expected:
                    self.tree.addValue.emit(relpath, size)
                else:
                    self.tree.addCount.emit(relpath, size)



#! /usr/bin/python3
# -*- coding: utf-8 -*-

#############################################################################
# Copyright (C) 2021 alexpdev
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
############################################################################
"""Setup package."""

import json

from setuptools import find_packages, setup

INFO = json.load(open("package.json"))
INFO["long_description"] = open("README.md").read()
with open("requirements.txt") as req:
    INFO["install_requires"] = req.read().split("\n")

setup(
    url=INFO["url"],
    name=INFO["name"],
    author=INFO["author"],
    license=INFO["license"],
    version=INFO["version"],
    keywords=INFO["keywords"],
    include_package_data=True,
    author_email=INFO["email"],
    description=INFO["description"],
    long_description=INFO["long_description"],
    packages=find_packages(exclude=["env", "tests"]),
    install_requires=["torrentfile", "PyQt6", "pyben"],
    project_urls={"Source Code": "https://github.com/alexpdev/torrentfileQt"},
    long_description_content_type="text/markdown",
    entry_points={
        "console_scripts": "torrentfileQt = torrentfileQt.window:start"
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: Apache Software License",
    ],
)



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""
Tab Widget containing all controls for creating a new .torrent file.

User must provide the path to the directory containing the what the
.torrent file will be created from.
"""
import os
import shutil
import subprocess  # nosec

from PyQt6.QtCore import Qt
from PyQt6.QtWidgets import (QCheckBox, QComboBox, QFileDialog, QGridLayout,
                             QHBoxLayout, QLabel, QLineEdit, QPlainTextEdit,
                             QPushButton, QRadioButton, QSpacerItem,
                             QToolButton, QWidget)
from torrentfile.utils import path_stat

from torrentfileQt.qss import pushButtonEdit


class CreateWidget(QWidget):
    """CreateWidget contains all controls for creating a new .torrent file.

    Args:
        QWidget (`QObject`): Parent class to CreateWidget.
    """

    def __init__(self, parent=None):
        """
        Constructor for Create Widget.

        Args:
            parent ([`QWidget`], optional): Parent Widget. Defaults to None.
        """
        super().__init__(parent=parent)
        self.content_dir = None
        self.outpath = None
        self.window = parent.window
        self.layout = QGridLayout()
        self.setLayout(self.layout)

        self.hlayout1 = QHBoxLayout()
        self.hlayout2 = QHBoxLayout()
        self.hlayout3 = QHBoxLayout()
        self.hlayout0 = QHBoxLayout()

        self.path_label = Label("Path: ", parent=self)
        self.output_label = Label("Save Path: ", parent=self)
        self.version_label = Label("Meta Version: ", parent=self)
        self.comment_label = Label("Comment: ", parent=self)
        self.announce_label = Label("Trackers: ", parent=self)
        self.source_label = Label("Source: ", parent=self)
        self.piece_length_label = Label("Piece Length: ", parent=self)

        self.path_input = LineEdit(parent=self)
        self.output_input = LineEdit(parent=self)
        self.source_input = LineEdit(parent=self)
        self.comment_input = LineEdit(parent=self)

        self.announce_input = PlainTextEdit(parent=self)
        self.piece_length = ComboBox(parent=self)
        self.private = CheckBox("Private", parent=self)
        self.submit_button = SubmitButton("Create Torrent", parent=self)
        self.browse_dir_button = BrowseDirButton(parent=self)
        self.browse_file_button = BrowseFileButton(parent=self)
        self.output_button = OutButton(parent=self)
        self.v1button = QRadioButton("v1 (default)", parent=self)
        self.v1button.setChecked(True)

        self.v2button = QRadioButton("v2", parent=self)
        self.hybridbutton = QRadioButton("v1+2 (hybrid)", parent=self)
        self.spacer = QSpacerItem(50, 0)

        self.path_label.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.path_input.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.output_label.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.output_input.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.version_label.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.source_label.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.source_input.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.comment_label.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.comment_input.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.announce_label.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.piece_length_label.setAlignment(Qt.AlignmentFlag.AlignLeft)

        self.hlayout0.addWidget(self.v1button)
        self.hlayout0.addWidget(self.v2button)
        self.hlayout0.addWidget(self.hybridbutton)
        self.hlayout1.addWidget(self.browse_dir_button)
        self.hlayout1.addWidget(self.browse_file_button)
        self.hlayout2.addWidget(self.piece_length)
        self.hlayout2.addItem(self.spacer)
        self.hlayout2.addWidget(self.private)
        self.hlayout3.addWidget(self.output_input)
        self.hlayout3.addWidget(self.output_button)

        self.layout.addWidget(self.path_label, 0, 0, 2, 1)
        self.layout.addWidget(self.path_input, 0, 1, 1, 3)
        self.layout.addLayout(self.hlayout1, 1, 1, 1, 3)
        self.layout.addWidget(self.version_label, 4, 0, 1, 1)
        self.layout.addLayout(self.hlayout0, 4, 1, 1, 3)
        self.layout.addWidget(self.piece_length_label, 3, 0, 1, 1)
        self.layout.addLayout(self.hlayout2, 3, 1, 1, 3)
        self.layout.addWidget(self.output_label, 2, 0, 1, 1)
        self.layout.addLayout(self.hlayout3, 2, 1, 1, 3)
        self.layout.addWidget(self.source_label, 5, 0, 1, 1)
        self.layout.addWidget(self.source_input, 5, 1, 1, 3)
        self.layout.addWidget(self.comment_label, 6, 0, 1, 1)
        self.layout.addWidget(self.comment_input, 6, 1, 1, 3)
        self.layout.addWidget(self.announce_label, 7, 0, 1, 1)
        self.layout.addWidget(self.announce_input, 7, 1, 1, 3)
        self.layout.addWidget(self.submit_button, 8, 0, 1, 4)
        for i in range(1, self.layout.columnCount()):
            self.layout.setColumnStretch(i, 1)
        self.layout.setObjectName("createWidget_formLayout")
        self.hlayout2.setObjectName("createWidget_hlayout2")
        self.submit_button.setObjectName("createWidget_submit_button")
        self.private.setObjectName("createWidget_private")
        self.path_label.setObjectName("createWidget_path_label")
        self.path_input.setObjectName("createWidget_path_input")
        self.piece_length.setObjectName("createWidget_piece_length")
        self.piece_length_label.setObjectName("createWidgetPiece_lengthLabel")
        self.source_label.setObjectName("createWidget_source_label")
        self.source_input.setObjectName("createWidget_source_input")
        self.announce_input.setObjectName("createWidget_announce_input")
        self.announce_label.setObjectName("createWidget_announce_label")
        self.comment_input.setObjectName("createWidget_comment_input")
        self.comment_label.setObjectName("createWidget_comment_label")
        self.browse_dir_button.setObjectName("createWidget_browsedir_button")
        self.browse_file_button.setObjectName("createWidget_browsefile_button")


def torrentfile_create(args, obj):  # pragma: no cover
    """
    Create new .torrent file in a seperate thread.

    Args:
        args ([`dict`]): keyword arguements for the torrent creator.
        obj ([`torrentfile.MetaBase`]): The procedure class for creating file.
    """
    tfile = obj(**args)
    tfile.write()


def create_torrent(args):
    """Create torrent file in seperate process."""
    tfexe = shutil.which("torrentfile")
    result = subprocess.run([tfexe, *args])  # nosec
    return result


class SubmitButton(QPushButton):
    """Button widget."""

    def __init__(self, text, parent=None):
        """Public Constructor for Submit Button.

        Args:
            text (str): Text displayed on the button itself.
            parent (QWidget, optional): This Widget's parent. Defaults None.
        """
        super().__init__(text, parent=parent)
        self._text = text
        self.widget = parent
        self.window = parent.window
        self.setCursor(Qt.CursorShape.PointingHandCursor)
        self.setText(text)
        self.pressed.connect(self.submit)

    def submit(self):
        """Submit Action performed when user presses Submit Button."""
        # Gather Information from other Widgets.
        args = []
        if self.widget.private.isChecked():
            args.append("--private")

        # add source to metadata
        sourcetext = self.widget.source_input.text()
        if sourcetext:
            args.extend(["--source", sourcetext])

        # add comments to metadata
        commenttext = self.widget.comment_input.text()
        if commenttext:
            args.extend(["--comment", commenttext])

        # at least 1 tracker input is required
        announce = self.widget.announce_input.toPlainText()
        announce = [i for i in announce.split("\n") if i]
        if announce:
            args.append("-a")
            args.extend(announce)

        # Calculates piece length if not specified by user.
        outtext = os.path.realpath(self.widget.output_input.text())
        if outtext:
            args.extend(["-o", outtext])

        current = self.widget.piece_length.currentIndex()
        if current:
            piece_length_index = self.widget.piece_length.currentIndex()
            piece_length = self.widget.piece_length.itemData(piece_length_index)
            args.extend(["--piece-length", str(piece_length)])

        if self.widget.hybridbutton.isChecked():
            args.extend(["--meta-version", "3"])
        elif self.widget.v2button.isChecked():
            args.extend(["--meta-version", "2"])

        path = self.widget.path_input.text()
        args.append(path)
        result = create_torrent(args)
        if result:
            self.window.statusBar().showMessage("Success")


class OutButton(QToolButton):
    """Button widget."""

    def __init__(self, parent=None):
        """Constructor for file picker for outfile button."""
        super().__init__(parent=parent)
        self.window = parent
        self.setText("...")
        self.setCursor(Qt.CursorShape.PointingHandCursor)
        self.pressed.connect(self.output)

    def output(self, outpath=None):
        """Assign output path for created torrent file."""
        caption = "Select Output Directory"
        if not outpath:  # pragma: no cover
            outpath = QFileDialog.getExistingDirectory(
                parent=self, caption=caption
            )
        if outpath:
            self.window.output_input.clear()
            self.parent().output_input.insert(outpath)
            self.parent().outpath = outpath


class BrowseFileButton(QPushButton):
    """Button widget for browse."""

    def __init__(self, parent=None):
        """Public constructor for browsebutton class."""
        super().__init__(parent=parent)
        self.setText("Select File")
        self.window = parent
        self.setStyleSheet(pushButtonEdit)
        self.setCursor(Qt.CursorShape.PointingHandCursor)
        self.pressed.connect(self.browse)

    def browse(self, path=None):
        """Browse Action performed when user presses button.

        Opens File/Folder Dialog.

        Returns:
            str: Path to file or folder to include in torrent.
        """
        caption = "Choose File"
        if not path:  # pragma: no cover
            path = QFileDialog.getOpenFileName(parent=self, caption=caption)
        if not isinstance(path, str):  # pragma: no cover
            path = os.path.normpath(path[0])
        else:
            path = os.path.normpath(path)
        self.window.path_input.clear()
        self.window.path_input.insert(path)
        self.window.output_input.clear()
        outdir = os.path.dirname(str(path))
        txt = os.path.split(str(path))[-1]
        outfile = os.path.splitext(txt)[0] + ".torrent"
        outpath = os.path.realpath(os.path.join(outdir, outfile))
        self.window.output_input.insert(outpath)
        _, size, piece_length = path_stat(path)
        if piece_length < (2 ** 20):
            val = f"{piece_length//(2**10)}KB"
        else:
            val = f"{piece_length//(2**20)}MB"  # pragma: no cover
        for i in range(self.window.piece_length.count()):
            if self.window.piece_length.itemText(i) == val:
                self.window.piece_length.setCurrentIndex(i)
                break
        return size


class BrowseDirButton(QPushButton):
    """Browse filesystem folders for path."""

    def __init__(self, parent=None):
        """Constructor for folder browser button."""
        super().__init__(parent=parent)
        self.setText("Select Folder")
        self.window = parent
        self.setStyleSheet(pushButtonEdit)
        self.pressed.connect(self.browse)
        self.setCursor(Qt.CursorShape.PointingHandCursor)

    def browse(self, filename=None):
        """Browse action performed when user presses button.

        Opens File/Folder Dialog.

        Returns:
            str: Path to file or folder to include in torrent.
        """
        caption = "Choose Root Directory"
        if not filename:  # pragma: no cover
            filename = QFileDialog.getExistingDirectory(
                parent=self, caption=caption
            )
        if filename:
            filename = os.path.realpath(filename)
            self.window.path_input.clear()
            self.window.path_input.insert(filename)
            self.window.output_input.clear()
            outdir = os.path.dirname(str(filename))
            outfile = (
                os.path.splitext(os.path.split(str(filename))[-1])[0]
                + ".torrent"
            )
            outpath = os.path.realpath(os.path.join(outdir, outfile))
            self.window.output_input.insert(outpath)
            try:
                _, _, piece_length = path_stat(filename)
            except PermissionError:  # pragma: no cover
                return
            if piece_length < (2 ** 20):
                val = f"{piece_length//(2**10)}KB"
            else:  # pragma: no cover
                val = f"{piece_length//(2**20)}MB"
            for i in range(self.window.piece_length.count()):
                if self.window.piece_length.itemText(i) == val:
                    self.window.piece_length.setCurrentIndex(i)
                    break
        return


class ComboBox(QComboBox):
    """Combo box options for selecting piece length."""

    def __init__(self, parent=None):
        """Constructor for ComboBox."""
        super().__init__(parent=parent)
        # self.setStyleSheet(comboBoxSheet)
        self.addItem("")
        for exp in range(14, 24):
            if exp < 20:
                item = str((2 ** exp) // (2 ** 10)) + "KB"
            else:
                item = str((2 ** exp) // (2 ** 20)) + "MB"
            self.addItem(item, 2 ** exp)
        self.setEditable(False)


class CheckBox(QCheckBox):
    """Checkbox widget."""

    def __init__(self, label, parent=None):
        """Constructor for check box widgit."""
        super().__init__(label, parent=parent)
        # self.setStyleSheet(checkBoxSheet)


class PlainTextEdit(QPlainTextEdit):
    """Text edit widget for trackers."""

    def __init__(self, parent=None):
        """Constructor for plain text edit."""
        super().__init__(parent=parent)
        self.setBackgroundVisible(True)
        # self.setStyleSheet(textEditSheet)


class Label(QLabel):
    """Label Identifier for Window Widgets.

    Subclass: QLabel
    """

    def __init__(self, text, parent=None):
        """Constructor for label widgit."""
        super().__init__(text, parent=parent)
        # self.setStyleSheet(labelSheet)
        font = self.font()
        font.setBold(True)
        font.setPointSize(12)
        self.setFont(font)


class LineEdit(QLineEdit):
    """Line edit widget for input fields."""

    def __init__(self, parent=None):
        """Constructor for line edit widget."""
        super().__init__(parent=parent)
        self.window = parent.window
        # self.setStyleSheet(createLineEditSheet)
        font = self.font()
        font.setPointSize(11)
        self.setFont(font)
